@@ -74,21 +74,23 @@ static char cmd_optstr[256];
 static int did_arg;
 
 const fio_fp64_t def_percentile_list[FIO_IO_U_LIST_MAX_LEN] = {
-	{ .u.f	=  1.0 },
-	{ .u.f	=  5.0 },
-	{ .u.f	= 10.0 },
-	{ .u.f	= 20.0 },
-	{ .u.f	= 30.0 },
-	{ .u.f	= 40.0 },
-	{ .u.f	= 50.0 },
-	{ .u.f	= 60.0 },
-	{ .u.f	= 70.0 },
-	{ .u.f	= 80.0 },
-	{ .u.f	= 90.0 },
-	{ .u.f	= 95.0 },
-	{ .u.f	= 99.0 },
-	{ .u.f	= 99.5 },
-	{ .u.f	= 99.9 },
+	{ .u.f	=  1.00 },
+	{ .u.f	=  5.00 },
+	{ .u.f	= 10.00 },
+	{ .u.f	= 20.00 },
+	{ .u.f	= 30.00 },
+	{ .u.f	= 40.00 },
+	{ .u.f	= 50.00 },
+	{ .u.f	= 60.00 },
+	{ .u.f	= 70.00 },
+	{ .u.f	= 80.00 },
+	{ .u.f	= 90.00 },
+	{ .u.f	= 95.00 },
+	{ .u.f	= 99.00 },
+	{ .u.f	= 99.50 },
+	{ .u.f	= 99.90 },
+	{ .u.f	= 99.95 },
+	{ .u.f	= 99.99 },
 };
 
 #define FIO_CLIENT_FLAG		(1 << 16)
@@ -568,6 +570,15 @@ static int fixup_options(struct thread_data *td)
 	}
 #endif
 
+	/*
+	 * For fully compressible data, just zero them at init time.
+	 * It's faster than repeatedly filling it.
+	 */
+	if (td->o.compress_percentage == 100) {
+		td->o.zero_buffers = 1;
+		td->o.compress_percentage = 0;
+	}
+
 	return ret;
 }
 
